--[[----------------------------------------------------------------------------

LrDocStoreTypeMigrator.lua

--------------------------------------------------------------------------------

Copyright 2020 Adobe
All Rights Reserved.

NOTICE: All information contained herein is, and remains
the property of Adobe and its suppliers, if any. The intellectual
and technical concepts contained herein are proprietary to Adobe
and its suppliers and are protected by all applicable intellectual
property laws, including trade secret and copyright laws
Dissemination of this information or reproduction of this material
is strictly forbidden unless prior written permission is obtained
from Adobe.

------------------------------------------------------------------------------]]

	-- WF.core.utils
local WFArguments = require 'WFArguments'

	-- LrDocStoreMigrator
local LrDocStoreTypeMigratorImp = require 'LrDocStoreTypeMigratorImp'

local WFCapabilities = require 'WFCapabilities'

--==============================================================================

local check = assert( WFArguments.check )

--------------------------------------------------------------------------------

local LrDocStoreTypeMigrator = {}

--------------------------------------------------------------------------------

function LrDocStoreTypeMigrator.docsWithTypeBackFillRequired( wfDocStoreImp )

	check( 'WFRxDocStoreImp' )

	if WFCapabilities.getCapability( "WFRxDocStore.enableLrDocStoreTypeMigrator" ) ~= "true" then
		return 0
	end

	-- In memory or temp docStores do not need a type/subtype back fill, neither does the
	-- oz-user docStore. Restrict the back fill to actual on disk "Managed Catalog"
	-- type docStores
	if not string.find( wfDocStoreImp:path(), "Managed Catalog") then
		return 0
	end

	-- Check the type column of the document with the highest local docId for NULL value.
	-- If it is NULL we need to back fill the type and subtype columns of all docs.
	local maxLocalDocIdTypeRow = wfDocStoreImp._db:execOnce(
		"SELECT type, localDocId FROM docs WHERE localdocId = (SELECT MAX( localDocId ) FROM docs )" )

	if maxLocalDocIdTypeRow and maxLocalDocIdTypeRow.type == nil then
		return maxLocalDocIdTypeRow.localDocId
	end

	return 0

end

--------------------------------------------------------------------------------


function LrDocStoreTypeMigrator.backFillTypeAndSubtype( wfDocStore )

	check( 'WFRxDocStoreImp' )

	return LrDocStoreTypeMigratorImp.backFillTypeAndSubtype( wfDocStore._db._db )

end

--------------------------------------------------------------------------------

return LrDocStoreTypeMigrator
