--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2020 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Inc.

----------------------------------------------------------------------------]]--

local KSData = require 'KSData'
local WFAsyncFileUtils = require "WFAsyncFileUtils"
local OzFilePaths = require "OzFilePaths"
local WFCrypto = require "WFCrypto"
local WFArguments = require 'WFArguments'
local check = WFArguments.check

----------------------------------------------------------------------

local LrmCatWatermark = {}

----------------------------------------------------------------------

function LrmCatWatermark.saveImageWatermarkCallbackFunc( wfCatalog, watermarkKSData, docValue )
	check( "WFCatalog", "KSData", "table" )

	local relativePath, absolutePath = OzFilePaths.getWatermarkGraphicPath( wfCatalog:ozCatalogId(), docValue, true )

	return WFAsyncFileUtils.saveToFile( watermarkKSData, absolutePath )
		:flatMap ( function( success )
			if success then
				return WFCrypto.asyncDigestFile( absolutePath, 'SHA256' )
					:map( function( sha256 )
						return relativePath, KSData.length( watermarkKSData ), sha256
					end )
			else
				error( string.format("failed to save watermark graphic to %s", absolutePath) )
			end
		end)

end

--------------------------------------------------------------------------------


return LrmCatWatermark
