--[[----------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2015 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

----------------------------------------------------------------------------]]--

local Rx = require "Rx"


	-- WF.lrmobile
local LrmCatCollaboration = require 'LrmCatCollaboration'

--------------------------------------------------------------------------------

local LrmLibrarySocialModelImp = {}

--==============================================================================

function LrmLibrarySocialModelImp:addComment( assetId, comment, albumId )

	local result = LrmCatCollaboration.addComment( self.catalog, albumId, assetId, comment )
		:map( function( data )
			return { state = true }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
	
	return {
		value = result,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:deleteComment( commentId )

	local result = self.catalog:docStore():deleteDoc( commentId )
		:map( function( data )
			return { state = true }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
		
	return {
		value = result,
	}

end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:deleteSpaceFavorite( spacefavoriteId )

	local result = self.catalog:docStore():deleteDoc( spacefavoriteId )
		:map( function( data )
			return { state = true }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
		
	return {
		value = result,
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:toggleMyFavorite( assetId, albumId )

	local result = LrmCatCollaboration.toggleMySpaceFavorite( self.catalog, assetId, albumId )
		:map( function( data )
			return { state = true }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
		
	return {
		value = result
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:shareAlbum( albumId, share )

	local changeSharingState = nil
	
	if share then
		changeSharingState = LrmCatCollaboration.shareAlbum( self.catalog, albumId )
	else
		changeSharingState = LrmCatCollaboration.unshareAlbum( self.catalog, albumId )
	end
	
	local result = changeSharingState
		:map( function( data )
			return { state = true }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
		
	
	return {
		value = result,
	}
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:resolveAlbumFromSpace( spaceId )

	local result LrmCatCollaboration.resolveAlbumFromSpace( self.catalog, spaceId )
		:map( function( data )
			return { state = true }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
		
	return {
		value = result
	}
	
end

--------------------------------------------------------------------------------

function LrmLibrarySocialModelImp:getSharedAlbumUrl( albumId )

	local result = LrmCatCollaboration.sharedAlbumUrl( self.catalog, albumId, true )
		:map( function( shareUrl )		
			return { state = true, sharedAlbumUrl = shareUrl }
		end )
		:catch( function( data )
			return Rx.Observable.just( { state = false, error = data } )
		end )
	
	return {
		value = result,
	}
end

--------------------------------------------------------------------------------

return LrmLibrarySocialModelImp
