--[[----------------------------------------------------------------------------

WFSQLiteCorruptionLoggerImp.lua

--------------------------------------------------------------------------------

ADOBE CONFIDENTIAL
------------------
Copyright 2018 Adobe Systems Incorporated
All Rights Reserved.

NOTICE: All information contained herein is, and remain the property of
Adobe Systems Incorporated and its suppliers, if any. The intellectual and
technical concepts contained herein are proprietary to Adobe Systems
Incorporated and its suppliers and may be covered by U.S. and Foreign
Patents, patents in process, and are protected by trade secret or copyright
law.  Dissemination of this information or reproduction of this material is
strictly forbidden unless prior written permission is obtained from
Adobe Systems Incorporated.

------------------------------------------------------------------------------]]

	-- WF.core.lua.threading
local WFCurrentThread = require 'WFCurrentThread'
local WFRequestDispatcher = require 'WFRequestDispatcher'

--------------------------------------------------------------------------------

local Rx = require "Rx"

--------------------------------------------------------------------------------

local _corruptionEventStream = Rx.Subject()

--------------------------------------------------------------------------------

local logger = require "WFLogger"( "WFSQLiteCorruptionLoggerImp" )

--------------------------------------------------------------------------------

local function logCorruptionEvent( errorType, path, msg, sql )

	logger:infof( "Saw corruption event: %s, %s, %s, %s", 
					errorType, path, msg, sql )

	_corruptionEventStream
		:onNext( errorType, path, msg, sql )

end

--------------------------------------------------------------------------------

local function maybeDispatchExternalMessage( notifName, ... )

	if notifName == "logCorruptionEvent" then

		local packedArgs = select( 1, ... )

		return logCorruptionEvent( unpack( packedArgs ) )

	end

end

--------------------------------------------------------------------------------

local function getCorruptionEventStream()

	return _corruptionEventStream

end

--------------------------------------------------------------------------------

local function workerThreadIsActiveStream()

	return Rx.Observable.just( true )

end

--------------------------------------------------------------------------------

local function initMessageProcessor()

	WFCurrentThread.registerExternalMessageDispatchHandler(
		maybeDispatchExternalMessage )

	WFRequestDispatcher.onMessage( 'logCorruptionEvent', logCorruptionEvent )

	WFRequestDispatcher.onMessage( 'getCorruptionEventStream', 
									getCorruptionEventStream )

	WFRequestDispatcher.onMessage( 'workerThreadIsActiveStream', 
									workerThreadIsActiveStream )
end

--==============================================================================

return {

	initMessageProcessor = initMessageProcessor,

}
